@@ -1,11 +1,14 @@
 """
 Author: masakokh
-Version: 3.1.0
+Version: 4.0.0
 """
 import datetime
 import os
 from datetime import datetime, timedelta
 from typing import Any
+# internal
+# from FCMLib import FCMLib
+from RedisLib import RedisLib
 
 
 class Logger:
@@ -17,7 +20,7 @@ class Logger:
 	# index of output
 	id      = 0
 
-	def __init__(self, path: str, filename: str, extension: str, enableLog: bool= True, enableConsole: bool= True, line: bool= True, color: bool= True):
+	def __init__(self, path: str= 'log', filename: str= 'access', extension: str= '.log', enableLog: bool= True, enableConsole: bool= True, line: bool= True, color: bool= True):
 		"""
 
 		:param path:
@@ -28,28 +31,42 @@ class Logger:
 		:param line:
 		:param color:
 		"""
-		# default datetime format
-		# 2020-05-18
+		# datetime format
+		# ex: 2020-05-18
 		self.__formatFileName   = '%Y-%m-%d'
-		# 2022-05-12 21:40:20.345
+		# full datetime format
+		# ex: 2022-05-12 21:40:20.345
 		self.__dateTimeFormat   = f'{self.__formatFileName} %H:%M:%S.%f'
 		# set color
 		self.__color            = color
 		self.__line             = line
 		# config
-		self.__enableLog        = enableLog
 		self.__enableConsole    = enableConsole
+		self.__enableLog        = enableLog
+		# redis
+		self.__enableRedis			= False
+		self.__enableRedisError		= False
+		self.__enableRedisFail		= False
+		self.__enableRedisInfo		= False
+		self.__enableRedisSuccess	= False
+		self.__enableRedisTrack		= False
+		self.__enableRedisWarning	= False
+		#
 		self.__extension        = extension
 		# path + /
 		self.__path             = path
-
 		# compute
-		self.__filename         = f'{self.__path}{filename}{self.__extension}'
-		#
-		self.__keySeries        = ''
+		self.__filename         = os.path.join(self.__path, f'{filename}{self.__extension}')
+		# static datetime
 		self.__datetime         = datetime.now().strftime(self.__dateTimeFormat)
+		# series
+		self.__keySeries        = ''
 		# session as uuid or md5
 		self.__keySession       = ''
+		# pub/sub
+		# self.__fcm				= FCMLib()
+		self.__redis			= RedisLib()
+		# load
 
 	def __createNewBackupFile(self) -> None:
 		"""
@@ -76,7 +93,7 @@ class Logger:
 		# generate a filename
 		return f'{self.__path}{(datetime.now() - timedelta(1)).strftime(self.__formatFileName)}{self.__extension}'
 
-	def __getContentBody(self, typeName: str, title: str, content: str, color: str) -> str:
+	def __getContentBody(self, typeName: str, title: str, content: Any, color: str) -> str:
 		"""
 
 		:param typeName:
@@ -164,6 +181,41 @@ class Logger:
 		else:
 			return ''
 
+	async def __msgRedis(self, channel: str= None) -> None:
+		"""
+
+		:param channel:
+		:return:
+		"""
+		self.__redis.messageGet()
+
+	# def __pushFCM(self, title: str, body: str) -> None:
+	# 	"""
+	#
+	# 	:param title:
+	# 	:param body:
+	# 	:return:
+	# 	"""
+	# 	pass
+
+	async def __pushRedis(self, title: str, body: str, channel: str= None) -> None:
+		"""
+
+		:param title:
+		:param body:
+		:param channel:
+		:return:
+		"""
+		#
+		notification	= self.__redis.messagePush(
+			title		= title
+			, body		= body
+			, channel	= channel
+		)
+
+		#
+		print(notification)
+
 	def __setNewId(self, id: int) -> None:
 		"""
 
@@ -173,7 +225,7 @@ class Logger:
 		if id > Logger.id:
 			Logger.id   = id
 
-	def __write(self, typeName: str = '', title: str = '', color: str = '', content: dict = None, logId: int = None) -> None:
+	def __write(self, typeName: str = '', title: str = '', color: str = '', content: Any = None, logId: int = None) -> None:
 		"""
 
 		:param typeName:
@@ -266,7 +318,7 @@ class Logger:
 		if numbers:
 			Logger.hide     = numbers
 
-	def error(self, title: str = '', content: dict = None, id: int = None) -> None:
+	def error(self, title: str = '', content: Any = None, id: int = None) -> None:
 		"""
 
 		:param title:
@@ -277,12 +329,12 @@ class Logger:
 		self.__write(
 			typeName    = 'ERROR'
 			, title     = title
-			, content   = content if content else {}
+			, content   = content
 			, color     = self.__StyleModifier.RED
 			, logId     = id
 		)
 
-	def fail(self, title: str = '', content: dict = None, id: int = None) -> None:
+	def fail(self, title: str = '', content: Any= None, id: int = None) -> None:
 		"""
 
 		:param title:
@@ -290,16 +342,23 @@ class Logger:
 		:param id:
 		:return:
 		"""
-
 		self.__write(
 			typeName    = 'FAIL'
 			, title     = title
-			, content   = content if content else {}
+			, content   = content
 			, color     = self.__StyleModifier.MAGENTA
 			, logId     = id
 		)
 
-	def info(self, title: str = '', content: dict = None, id: int = None) -> None:
+		#
+		if self.__enableRedis and self.__redis.enableFail:
+			#
+			self.__redis.messagePush(
+				title	= title
+				, body	= content
+			)
+
+	def info(self, title: str = '', content: Any = None, id: int = None) -> None:
 		"""
 
 		:param title:
@@ -310,11 +369,27 @@ class Logger:
 		self.__write(
 			typeName    = 'INFO'
 			, title     = title
-			, content   = content if content else {}
+			, content   = content
 			, color     = self.__StyleModifier.BLUE
 			, logId     = id
 		)
 
+		#
+		if self.__enableRedis and self.__redis.enableInfo:
+			#
+			self.__redis.messagePush(
+				title	= title
+				, body	= content
+			)
+
+	# def setFCM(self, config: dict) -> None:
+	# 	"""
+	#
+	# 	:param config:
+	# 	:return:
+	# 	"""
+	# 	self.__fcm.setConfig(config)
+
 	def setKeySeries(self, series: str = None) -> None:
 		"""
 
@@ -326,17 +401,52 @@ class Logger:
 	def setKeySession(self, sessionKey: str= None) -> None:
 		"""
 
+		:note: set session filename
 		:param sessionKey:
 		:return:
 		"""
 		if sessionKey and len(sessionKey) > 32:
+			# start 0 to 31st of string
 			self.__keySession       = sessionKey[0:31]
 
 		else:
 			# accept even empty or none
+			# empty or none won't able this feature
 			self.__keySession       = sessionKey
 
-	def success(self, title: str = '', content: dict = None, id: int = None) -> None:
+	def setRedis(self, enable: bool, host: str, port: int, db: int= 0, password: str= None, enableFail: bool= False, enableInfo: bool= False, enableTrack: bool= False, enableSuccess: bool= False, enableWarning: bool= False, channel: str= None) -> None:
+		"""
+
+		:param enable:
+		:param host:
+		:param port:
+		:param db:
+		:param password:
+		:param channel:
+		:return:
+		"""
+		# assign value
+		self.__enableRedis		= enable
+
+		# config the redis property to connect to the server
+		self.__redis.config(
+			host		= host
+			, port		= port
+			, db		= db
+			, password	= password
+		)
+
+		# enable level of pub/sub
+		self.__redis.enableFail		= enableFail
+		self.__redis.enableInfo		= enableInfo
+		self.__redis.enableTrack	= enableTrack
+		self.__redis.enableSuccess	= enableSuccess
+		self.__redis.enableWarning	= enableWarning
+
+		# set channel name
+		self.__redis.channelSet(channel= channel)
+
+	def success(self, title: str = '', content: Any = None, id: int = None) -> None:
 		"""
 		:param title:
 		:param content:
@@ -346,12 +456,20 @@ class Logger:
 		self.__write(
 			typeName    = 'SUCCESS'
 			, title     = title
-			, content   = content if content else {}
+			, content   = content
 			, color     = self.__StyleModifier.GREEN
 			, logId     = id
 		)
 
-	def track(self, title: str = '', content: dict = None, id: int = None) -> None:
+		#
+		if self.__enableRedis and self.__redis.enableSuccess:
+			#
+			self.__redis.messagePush(
+				title	= title
+				, body	= content
+			)
+
+	def track(self, title: str = '', content: Any = None, id: int = None) -> None:
 		"""
 
 		:param title:
@@ -366,7 +484,15 @@ class Logger:
 			, logId     = id
 		)
 
-	def warning(self, title: str = '', content: dict = None, id: int = None) -> None:
+		#
+		if self.__enableRedis and self.__redis.enableTrack:
+			#
+			self.__redis.messagePush(
+				title	= title
+				, body	= content
+			)
+
+	def warning(self, title: str = '', content: Any = None, id: int = None) -> None:
 		"""
 
 		:param title:
@@ -377,11 +503,19 @@ class Logger:
 		self.__write(
 			typeName    = 'WARNING'
 			, title     = title
-			, content   = content if content else {}
+			, content   = content
 			, color     = self.__StyleModifier.YELLOW
 			, logId     = id
 		)
 
+		#
+		if self.__enableRedis and self.__redis.enableWarning:
+			#
+			self.__redis.messagePush(
+				title	= title
+				, body	= content
+			)
+
 	class __StyleModifier:
 		# Foreground
 		BLUE            = '\033[94m'
